#!/usr/bin/lua

local wmsldap = require"core.wmsldap"

function readFile(path)
	local content = ""
	local file, error = io.open(path , "r")
	if file then
	    content = file:read("*all")
	    io.close(file)
	end
	return content
end

function sendXmlRpc(method, data)
	if not method or (data and "table" ~= type(data)) then
        return nil
    end

    require"xmlrpc.http"
    local rv, respData = xmlrpc.http.call(
         "http://127.0.0.1:6789/RPC2",
         method,
         unpack(data or {})
    )
    return rv and respData
end


local currentDomain = readFile("/etc/hostname")..".wildixin.com";
local newXmppSrvRecords = ""
local newSipNaptrRecords = ""
local newSipSrvRecords = ""
local pbx = {}
local arrayOfPbxs = {}
local arrayOfFailovers = {}
local json = require"dkjson"
local needRestart = "no"
local prio = 0
local weight = 1
local publicPort = ""

local filter = '(objectClass=wPbx)'
local attrs ={"dn", "pbxIp", "pbxDomain", "description", "pbxPublicPort" }
local failoverFilter = '(objectClass=wFailover)'
local failoverAttrs ={"dn", "pbxowner", "pbxFailover", "priority", "enabled"}
local connection, basedn = wmsldap:getConection()

if connection ~= nil then
	for dn,attrs in connection:search { attrs = attrs, filter = filter, base = basedn, scope = "subtree"} do
		arrayOfPbxs[dn] = attrs
	end

	for dn,attrs in connection:search { attrs = failoverAttrs, filter = failoverFilter, base = basedn, scope = "subtree"} do
		arrayOfFailovers[dn] = attrs
	end
end
	connection:close()

for dn,attrs in pairs(arrayOfPbxs) do

	-- use custom secure port for tcp sip srv
		if attrs["pbxPublicPort"] ~= nil then
			publicPort = attrs["pbxPublicPort"]
		else
			publicPort = 5060
		end

		if  attrs["pbxDomain"] ~= currentDomain then
		pbx[attrs["pbxIp"]] = attrs["pbxDomain"]

			newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp."..attrs["pbxDomain"]..","..attrs["pbxIp"]..",5269,1,1\n"
			newXmppSrvRecords = newXmppSrvRecords.."srv-host=_xmpp-server._tcp.conference."..attrs["pbxDomain"]..","..attrs["pbxIp"]..",5269,1,1\n"
		end

-- sip naptr
		newSipNaptrRecords = newSipNaptrRecords.."### NAPTR for " .. attrs["pbxDomain"] .. " ### \n"
		newSipNaptrRecords = newSipNaptrRecords.."naptr-record="..attrs["pbxDomain"]..",1,0,S,SIP+D2U,,_sip._udp."..attrs["pbxDomain"].."\n"
		newSipNaptrRecords = newSipNaptrRecords.."naptr-record="..attrs["pbxDomain"]..",0,0,S,SIP+D2T,,_sip._tcp."..attrs["pbxDomain"].."\n"
		newSipNaptrRecords = newSipNaptrRecords.."\n"
-- sip srv
		prio = 1
		newSipSrvRecords = newSipSrvRecords.."### SRV for " ..attrs["pbxDomain"].. " ### \n"
		if  attrs["pbxDomain"] ~= currentDomain then
			newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..attrs["pbxDomain"]..","..attrs["pbxIp"]..",5060,"..prio..","..weight.."\n"
		end
		if attrs["description"] then
		  local a, b = string.match(attrs["description"], "(.*)%=(.*)")
		  local res = json.decode(b)
		  for key, ip in pairs(res["addrSet"]["list"]) do
			  prio = prio+1
			  newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..attrs["pbxDomain"]..","..ip..",5060,"..prio..","..weight.."\n"
		  end
		end
		prio = prio+1
		newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..attrs["pbxDomain"]..","..attrs["pbxDomain"]..",5060,"..prio..","..weight.."\n"
		newSipSrvRecords = newSipSrvRecords.."\n"

		prio = 1
		if  attrs["pbxDomain"] ~= currentDomain then
			newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..attrs["pbxDomain"]..","..attrs["pbxIp"]..","..publicPort..","..prio..","..weight.."\n"
		end
		if attrs["description"] then
  		local a, b = string.match(attrs["description"], "(.*)%=(.*)")
  		local res = json.decode(b)
  		for key, ip in pairs(res["addrSet"]["list"]) do
  			prio = prio+1
  			newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..attrs["pbxDomain"]..","..ip..","..publicPort..","..prio..","..weight.."\n"
  		end
    end
		prio = prio+1
		newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..attrs["pbxDomain"]..","..attrs["pbxDomain"]..","..publicPort..","..prio..","..weight.."\n"
		newSipSrvRecords = newSipSrvRecords.."\n"
end
-- sip failover srv
-- srv priority start from 1000 + priority getted from LDAP for current wFailover object
-- increased LDAP wFailover priority x100 to avoid double record with the same proirity(it means that allowed maximum 100 interfaces per PBX)
for pbxowner,attrs in pairs(arrayOfFailovers) do
	if arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]] ~= nil then

		-- use custom secure port for failover tcp sip srv
		if arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["pbxPublicPort"] ~= nil then
			publicPort = arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["pbxPublicPort"]
		else
			publicPort = 5060
		end

		local ifaceCounter = 0
		prio = 1000 + (attrs["priority"] * 100)
		ifaceCounter = ifaceCounter + 1
		newSipSrvRecords = newSipSrvRecords .."### Failover for " .. arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]]['pbxDomain'] .. " is: " .. arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]['pbxDomain'].." ### \n"
		newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]]['pbxDomain']..","..arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["pbxIp"]..",5060,"..prio..","..weight.."\n"
		local a, b = string.match(arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["description"], "(.*)%=(.*)")
		local res = json.decode(b)
		for key, ip in pairs(res["addrSet"]["list"]) do
			prio = prio + 1
			ifaceCounter = ifaceCounter + 1
			newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]]['pbxDomain']..","..ip..",5060,"..prio..","..weight.."\n"
		end
		prio = prio + 1
		ifaceCounter = ifaceCounter + 1
		newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._udp."..arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]]['pbxDomain']..","..arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["pbxDomain"]..",5060,"..prio..","..weight.."\n"
		newSipSrvRecords = newSipSrvRecords.."\n"

		prio = prio - ifaceCounter
		prio = prio + 1
		newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]]['pbxDomain']..","..arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["pbxIp"]..","..publicPort..","..prio..","..weight.."\n"
		local a, b = string.match(arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["description"], "(.*)%=(.*)")
		local res = json.decode(b)
		for key, ip in pairs(res["addrSet"]["list"]) do
			prio = prio + 1
			newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]]["pbxDomain"]..","..ip..","..publicPort..","..prio..","..weight.."\n"
		end
		prio = prio + 1
		newSipSrvRecords = newSipSrvRecords.."srv-host=_sip._tcp."..arrayOfPbxs[arrayOfFailovers[pbxowner]["pbxowner"]]["pbxDomain"]..","..arrayOfPbxs[arrayOfFailovers[pbxowner]['pbxFailover']]["pbxDomain"]..","..publicPort..","..prio..","..weight.."\n"
		newSipSrvRecords = newSipSrvRecords.."\n"
	end
end


local srvXmppRecords = readFile("/etc/dnsmasq.d/ejabberd_dns.conf")
local srvSipRecords = readFile("/etc/dnsmasq.d/sip_dns.conf")

if srvSipRecords ~= newSipNaptrRecords..newSipSrvRecords then
	local file, error = io.open("/etc/dnsmasq.d/sip_dns.conf", "w")
	file:write(newSipNaptrRecords .. newSipSrvRecords)
	io.close(file)
	needRestart = "yes"
end

if srvXmppRecords ~= newXmppSrvRecords then

	for domain, ip in srvXmppRecords:gmatch("srv%-host=_xmpp%-server._tcp.([^,]+),([^,]+),5269,1,1") do
	  if domain:find("conference.", 1, true) == nil then

	  	if pbx[ip] ~= nil then
			if pbx[ip] ~= domain then
				-- changed domain
				-- обновить историю чата
				-- print("changed domain from "..domain.." to "..pbx[ip])
				-- sendXmlRpc("host_changed", {{from = domain}, {to = pbx[ip]}})
			end
	  	else
	  		-- pbx removed
	  		-- sendXmlRpc("host_removed", {{host = domain}})
	  	end
	  end
	end

	local file, error = io.open("/etc/dnsmasq.d/ejabberd_dns.conf", "w")
	file:write(newXmppSrvRecords)
	io.close(file)
	needRestart = "yes"
end

if needRestart == "yes" then
	os.execute("sudo /etc/init.d/dnsmasq restart")
end
