(function universalModuleDefinition(root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS
        module.exports = factory(require('jquery'), require('WMS'));
    } else if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery', 'WMS'], factory);
    } else {
        // Browser
        root.CrmIntegrations = factory(root.jQuery, root.WMS);
    }
}(this, function ($, WMS) {
    'use strict';

    function CrmIntegrations(params) {
        if (typeof params === 'object') {
            this._settings = params;
        }
    }

    $.extend(CrmIntegrations.prototype, {
        _content: null,

        load: function () {
            this._htmlContainer = $('#crmIntegrationsWrapper');

            this._render();
            this._initForm();
        },

        _render: function () {
            var data = {
                'txt': {
                    'crm_integrations': {
                        'callback_urls': {
                            'title': WMS._t('crm_integrations.callback_urls.title'),
                            'new_call': WMS._t('crm_integrations.callback_urls.new_call'),
                            'call_updated': WMS._t('crm_integrations.callback_urls.call_updated'),
                            'call_ended': WMS._t('crm_integrations.callback_urls.call_ended'),
                            'basic_login': WMS._t('crm_integrations.callback_urls.basic_login'),
                            'basic_password': WMS._t('crm_integrations.callback_urls.basic_password'),
                            'verify_certificate': WMS._t('crm_integrations.callback_urls.verify_certificate')
                        },
                        'params_explain': WMS._t('crm_integrations.params_explain').replace(/[|]/g, '<br/>')
                    },
                    'buttons': {
                        'Save': WMS._t('buttons.Save')
                    }
                },
                'settings': this._settings
            };

            var content = $.tmpl('crm_integrations', data);
            this._htmlContainer.append(content);

            this._urlsForm = this._htmlContainer.find('#integration_crms_form');
        },

        _initForm: function () {
            this._urlsForm.apiForm({
                load: 'api/v1/PBX/settings/crm',
                submit: 'api/v1/PBX/settings/crm',

                error: function(response) {
                    setError('error', response.reason, 5000);
                },

                beforeSubmit: function() {
                    ap_showWaitMessage('waitDiv', 1);

                    return true;
                },

                afterSubmit: function(response) {
                    this._settings = response.result;
                    ap_showWaitMessage('waitDiv', 0);
                }.bind(this)
            });

            this._setEvents();
        },

        _setEvents: function () {
            this._urlsForm.on('click', '.crmCallbackUrlsSave', this._onSaveUrlsButtonClick.bind(this));
        },

        _onSaveUrlsButtonClick: function () {
            this._urlsForm.apiForm('submit');
        }
    });

    return CrmIntegrations;
}));
